#!/usr/bin/env python
# 
# Copyright 2002, 2003 Zuza Software Foundation
# 
# This file is part of translate.
#
# translate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# translate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with translate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""tool to import a .po file into the database"""

from translate.portal import config, portaldb
from translate.storage import po
from jToolkit.data import database, dbtable
from jToolkit import errors
import sys

class poimporter:
  def __init__(self, pdb):
    self.pdb = pdb

  def importfile(self, pofile, language):
    origadded, origmodified = 0, 0
    transadded, transmodified = 0, 0
    fileid = self.pdb.addsourcefile(pofile.filename)
    print "fileid: %r" % fileid
    versionname = "thisversion"
    for thepo in pofile.poelements:
      (origadd, origmodify), (transadd, transmodify) = self.handlepoelement(language, fileid, versionname, thepo)
      origadded += origadd
      origmodified += origmodify
      transadded += transadd
      transmodified += transmodify
    return (origadded, origmodified, transadded, transmodified)

  def handlepoelement(self, language, fileid, versionname, thepo):
    versiontype = "import"
    username = ""
    identifier = " ".join(thepo.getsources())
    originaltext = po.getunquotedstr(thepo.msgid)
    translation = po.getunquotedstr(thepo.msgstr)
    origresults = self.pdb.addoriginaltext(filename, identifier, version, originaltext)
    transresults = self.pdb.addtranslation(language, filename, identifier, version, translation, username)
    return origresults, transresults

def getvalidconfigs():
  return ", ".join([configname for configname in dir(config) if not configname.startswith("_")])

def main(inputfile, dbconfig, language, verbose):
  inputpo = po.pofile(inputfile)
  if verbose:
    tracefile = sys.stdout
  else:
    tracefile = None
  db = database.dbwrapper(dbconfig, errors.ConsoleErrorHandler(tracefile=tracefile))
  pdb = portaldb.portaldb(db)
  importer = poimporter(pdb)
  origadded, origmodified, transadded, transmodified = importer.importfile(inputpo, language)
  print "added %d new original strings, %d new translations" % (origadded, transadded)
  print "modified %d original strings, %d translations" % (origmodified, transmodified)

if __name__ == '__main__':
  # handle command line options
  try:
    import optparse
  except ImportError:
    from translate.misc import optparse
  inputformat = "po"
  parser = optparse.OptionParser(usage="%prog -l|--language language [-i|--input-file inputfile] [-d|--dbname databasename] [-v|--verbose]")
  parser.add_option("-i", "--input-file", dest="inputfile", default=None,
                    help="read from inputfile in "+inputformat+" format", metavar="inputfile")
  parser.add_option("-d", "--dbname", dest="databasename", default="default",
                    help="use this database (valid options are: %s)" % getvalidconfigs(), metavar="databasename")
  parser.add_option("-l", "--language", dest="language", default="??",
                    help="default target language", metavar="language")
  parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="print trace messages to stdout")
  (options, args) = parser.parse_args()
  # open the appropriate files
  if options.inputfile is None:
    inputfile = sys.stdin
  else:
    inputfile = open(options.inputfile, 'r')
  dbconfig = getattr(config, options.databasename, None)
  if dbconfig is None:
    raise ValueError("could not find dbname %s: value options are %r" % (dbconfig, getvalidconfigs()))
  main(inputfile, dbconfig, options.language, options.verbose)

